/* Copyright (C) 2017-2018 Facility for Antiproton and Ion Research in Europe, Darmstadt
   SPDX-License-Identifier: GPL-3.0-only
   Authors: Pierre-Alain Loizeau [committer] */

/** @file Cern2017Monitor.C
 ** @author Pierre-Alain Loizeau <p.-a.loizeau@gsi.de>
 ** @date 26.07.2017
 **
 ** ROOT macro to read tsa files which have been produced with StsXyter + DPB + FLIB
 ** Convert data into cbmroot format.
 ** Uses CbmFlibTestSource as source task.
 */

// In order to call later Finish, we make this global
FairRunOnline* run = NULL;

void EfficiencyMonitor_kronos_nocalib(Int_t iRunIdx = 84)
{
  const Int_t kiNbRuns        = 89;
  TString sFilePath[kiNbRuns] = {"/lustre/nyx/cbm/prod/beamtime/2018/02/cosy/"
                                 "r0001_20180220_1034/r0001_20180220_1034",
                                 "/lustre/nyx/cbm/prod/beamtime/2018/02/cosy/"
                                 "r0002_20180220_1046/r0002_20180220_1046",
                                 "/lustre/nyx/cbm/prod/beamtime/2018/02/cosy/"
                                 "r0003_20180220_1413/r0003_20180220_1413",
                                 "/lustre/nyx/cbm/prod/beamtime/2018/02/cosy/"
                                 "r0004_20180222_1622/r0004_20180222_1622",
                                 "/lustre/nyx/cbm/prod/beamtime/2018/02/cosy/"
                                 "r0005_20180222_1636/r0005_20180222_1636",
                                 "/lustre/nyx/cbm/prod/beamtime/2018/02/cosy/"
                                 "r0006_20180222_1646/r0006_20180222_1646",
                                 "/lustre/nyx/cbm/prod/beamtime/2018/02/cosy/"
                                 "r0007_20180227_1058/r0007_20180227_1058",
                                 "/lustre/nyx/cbm/prod/beamtime/2018/02/cosy/"
                                 "r0008_20180227_1624/r0008_20180227_1624",
                                 "/lustre/nyx/cbm/prod/beamtime/2018/02/cosy/"
                                 "r0009_20180228_1146/r0009_20180228_1146",
                                 "/lustre/nyx/cbm/prod/beamtime/2018/02/cosy/"
                                 "r0010_20180228_1246/r0010_20180228_1246",
                                 "/lustre/nyx/cbm/prod/beamtime/2018/02/cosy/"
                                 "r0011_20180228_1753/r0011_20180228_1753",
                                 "/lustre/nyx/cbm/prod/beamtime/2018/02/cosy/"
                                 "r0012_20180228_1831/r0012_20180228_1831",
                                 "/lustre/nyx/cbm/prod/beamtime/2018/02/cosy/"
                                 "r0013_20180228_1905/r0013_20180228_1905",
                                 "/lustre/nyx/cbm/prod/beamtime/2018/02/cosy/"
                                 "r0014_20180301_0951/r0014_20180301_0951",
                                 "/lustre/nyx/cbm/prod/beamtime/2018/02/cosy/"
                                 "r0015_20180301_1011/r0015_20180301_1011",
                                 "/lustre/nyx/cbm/prod/beamtime/2018/02/cosy/"
                                 "r0016_20180301_1054/r0016_20180301_1054",
                                 "/lustre/nyx/cbm/prod/beamtime/2018/02/cosy/"
                                 "r0017_20180301_1116/r0017_20180301_1116",
                                 "/lustre/nyx/cbm/prod/beamtime/2018/02/cosy/"
                                 "r0018_20180301_1131/r0018_20180301_1131",
                                 "/lustre/nyx/cbm/prod/beamtime/2018/02/cosy/"
                                 "r0019_20180301_1149/r0019_20180301_1149",
                                 "/lustre/nyx/cbm/prod/beamtime/2018/02/cosy/"
                                 "r0020_20180301_1813/r0020_20180301_1813",
                                 "/lustre/nyx/cbm/prod/beamtime/2018/02/cosy/"
                                 "r0021_20180301_1836/r0021_20180301_1836",
                                 "/lustre/nyx/cbm/prod/beamtime/2018/02/cosy/"
                                 "r0022_20180301_1900/r0022_20180301_1900",
                                 "/lustre/nyx/cbm/prod/beamtime/2018/02/cosy/"
                                 "r0023_20180301_1923/r0023_20180301_1923",
                                 "/lustre/nyx/cbm/prod/beamtime/2018/02/cosy/"
                                 "r0024_20180301_1929/r0024_20180301_1929",
                                 "/lustre/nyx/cbm/prod/beamtime/2018/02/cosy/"
                                 "r0025_20180301_1938/r0025_20180301_1938",
                                 "/lustre/nyx/cbm/prod/beamtime/2018/02/cosy/"
                                 "r0026_20180301_2006/r0026_20180301_2006",
                                 "/lustre/nyx/cbm/prod/beamtime/2018/02/cosy/"
                                 "r0027_20180301_2056/r0027_20180301_2056",
                                 "/lustre/nyx/cbm/prod/beamtime/2018/02/cosy/"
                                 "r0028_20180301_2104/r0028_20180301_2104",
                                 "/lustre/nyx/cbm/prod/beamtime/2018/02/cosy/"
                                 "r0029_20180301_2120/r0029_20180301_2120",
                                 "/lustre/nyx/cbm/prod/beamtime/2018/02/cosy/"
                                 "r0030_20180301_2139/r0030_20180301_2139",
                                 "/lustre/nyx/cbm/prod/beamtime/2018/02/cosy/"
                                 "r0031_20180302_0810/r0031_20180302_0810",
                                 "/lustre/nyx/cbm/prod/beamtime/2018/02/cosy/"
                                 "r0032_20180302_0914/r0032_20180302_0914",
                                 "/lustre/nyx/cbm/prod/beamtime/2018/02/cosy/"
                                 "r0033_20180302_0923/r0033_20180302_0923",
                                 "/lustre/nyx/cbm/prod/beamtime/2018/02/cosy/"
                                 "r0034_20180302_1025/r0034_20180302_1025",
                                 "/lustre/nyx/cbm/prod/beamtime/2018/02/cosy/"
                                 "r0035_20180302_1247/r0035_20180302_1247",
                                 "/lustre/nyx/cbm/prod/beamtime/2018/02/cosy/"
                                 "r0036_20180302_1307/r0036_20180302_1307",
                                 "/lustre/nyx/cbm/prod/beamtime/2018/02/cosy/"
                                 "r0037_20180302_1320/r0037_20180302_1320",
                                 "/lustre/nyx/cbm/prod/beamtime/2018/02/cosy/"
                                 "r0038_20180302_1336/r0038_20180302_1336",
                                 "/lustre/nyx/cbm/prod/beamtime/2018/02/cosy/"
                                 "r0039_20180302_1437/r0039_20180302_1437",
                                 "/lustre/nyx/cbm/prod/beamtime/2018/02/cosy/"
                                 "r0040_20180302_1449/r0040_20180302_1449",
                                 "/lustre/nyx/cbm/prod/beamtime/2018/02/cosy/"
                                 "r0041_20180302_1503/r0041_20180302_1503",
                                 "/lustre/nyx/cbm/prod/beamtime/2018/02/cosy/"
                                 "r0042_20180302_1517/r0042_20180302_1517",
                                 "/lustre/nyx/cbm/prod/beamtime/2018/02/cosy/"
                                 "r0043_20180302_1532/r0043_20180302_1532",
                                 "/lustre/nyx/cbm/prod/beamtime/2018/02/cosy/"
                                 "r0044_20180302_1555/r0044_20180302_1555",
                                 "/lustre/nyx/cbm/prod/beamtime/2018/02/cosy/"
                                 "r0045_20180302_1642/r0045_20180302_1642",
                                 "/lustre/nyx/cbm/prod/beamtime/2018/02/cosy/"
                                 "r0046_20180302_1726/r0046_20180302_1726",
                                 "/lustre/nyx/cbm/prod/beamtime/2018/02/cosy/"
                                 "r0047_20180302_2041/r0047_20180302_2041",
                                 "/lustre/nyx/cbm/prod/beamtime/2018/02/cosy/"
                                 "r0048_20180303_0852/r0048_20180303_0852",
                                 "/lustre/nyx/cbm/prod/beamtime/2018/02/cosy/"
                                 "r0049_20180303_0900/r0049_20180303_0900",
                                 "/lustre/nyx/cbm/prod/beamtime/2018/02/cosy/"
                                 "r0050_20180303_0918/r0050_20180303_0918",
                                 "/lustre/nyx/cbm/prod/beamtime/2018/02/cosy/"
                                 "r0051_20180303_0945/r0051_20180303_0945",
                                 "/lustre/nyx/cbm/prod/beamtime/2018/02/cosy/"
                                 "r0052_20180303_1006/r0052_20180303_1006",
                                 "/lustre/nyx/cbm/prod/beamtime/2018/02/cosy/"
                                 "r0053_20180303_1022/r0053_20180303_1022",
                                 "/lustre/nyx/cbm/prod/beamtime/2018/02/cosy/"
                                 "r0054_20180303_1033/r0054_20180303_1033",
                                 "/lustre/nyx/cbm/prod/beamtime/2018/02/cosy/"
                                 "r0055_20180303_1048/r0055_20180303_1048",
                                 "/lustre/nyx/cbm/prod/beamtime/2018/02/cosy/"
                                 "r0056_20180303_1100/r0056_20180303_1100",
                                 "/lustre/nyx/cbm/prod/beamtime/2018/02/cosy/"
                                 "r0057_20180303_1120/r0057_20180303_1120",
                                 "/lustre/nyx/cbm/prod/beamtime/2018/02/cosy/"
                                 "r0058_20180303_1140/r0058_20180303_1140",
                                 "/lustre/nyx/cbm/prod/beamtime/2018/02/cosy/"
                                 "r0059_20180303_1714/r0059_20180303_1714",
                                 "/lustre/nyx/cbm/prod/beamtime/2018/02/cosy/"
                                 "r0060_20180303_1903/r0060_20180303_1903",
                                 "/lustre/nyx/cbm/prod/beamtime/2018/02/cosy/"
                                 "r0061_20180303_1944/r0061_20180303_1944",
                                 "/lustre/nyx/cbm/prod/beamtime/2018/02/cosy/"
                                 "r0062_20180303_2035/r0062_20180303_2035",
                                 "/lustre/nyx/cbm/prod/beamtime/2018/02/cosy/"
                                 "r0063_20180303_2115/r0063_20180303_2115",
                                 "/lustre/nyx/cbm/prod/beamtime/2018/02/cosy/"
                                 "r0064_20180303_2134/r0064_20180303_2134",
                                 "/lustre/nyx/cbm/prod/beamtime/2018/02/cosy/"
                                 "r0065_20180303_2157/r0065_20180303_2157",
                                 "/lustre/nyx/cbm/prod/beamtime/2018/02/cosy/"
                                 "r0066_20180303_2211/r0066_20180303_2211",
                                 "/lustre/nyx/cbm/prod/beamtime/2018/02/cosy/"
                                 "r0067_20180303_2224/r0067_20180303_2224",
                                 "/lustre/nyx/cbm/prod/beamtime/2018/02/cosy/"
                                 "r0068_20180303_2244/r0068_20180303_2244",
                                 "/lustre/nyx/cbm/prod/beamtime/2018/02/cosy/"
                                 "r0069_20180304_0813/r0069_20180304_0813",
                                 "/lustre/nyx/cbm/prod/beamtime/2018/02/cosy/"
                                 "r0070_20180304_0847/r0070_20180304_0847",
                                 "/lustre/nyx/cbm/prod/beamtime/2018/02/cosy/"
                                 "r0071_20180304_0905/r0071_20180304_0905",
                                 "/lustre/nyx/cbm/prod/beamtime/2018/02/cosy/"
                                 "r0072_20180304_0930/r0072_20180304_0930",
                                 "/lustre/nyx/cbm/prod/beamtime/2018/02/cosy/"
                                 "r0073_20180304_0956/r0073_20180304_0956",
                                 "/lustre/nyx/cbm/prod/beamtime/2018/02/cosy/"
                                 "r0074_20180304_1019/r0074_20180304_1019",
                                 "/lustre/nyx/cbm/prod/beamtime/2018/02/cosy/"
                                 "r0075_20180304_1041/r0075_20180304_1041",
                                 "/lustre/nyx/cbm/prod/beamtime/2018/02/cosy/"
                                 "r0076_20180304_1056/r0076_20180304_1056",
                                 "/lustre/nyx/cbm/prod/beamtime/2018/02/cosy/"
                                 "r0077_20180304_1111/r0077_20180304_1111",
                                 "/lustre/nyx/cbm/prod/beamtime/2018/02/cosy/"
                                 "r0078_20180304_1130/r0078_20180304_1130",
                                 "/lustre/nyx/cbm/prod/beamtime/2018/02/cosy/"
                                 "r0079_20180304_1149/r0079_20180304_1149",
                                 "/lustre/nyx/cbm/prod/beamtime/2018/02/cosy/"
                                 "r0080_20180304_1429/r0080_20180304_1429",
                                 "/lustre/nyx/cbm/prod/beamtime/2018/02/cosy/"
                                 "r0081_20180304_1456/r0081_20180304_1456",
                                 "/lustre/nyx/cbm/prod/beamtime/2018/02/cosy/"
                                 "r0082_20180304_1504/r0082_20180304_1504",
                                 "/lustre/nyx/cbm/prod/beamtime/2018/02/cosy/"
                                 "r0083_20180304_1517/r0083_20180304_1517",
                                 "/lustre/nyx/cbm/prod/beamtime/2018/02/cosy/"
                                 "r0084_20180304_1627/r0084_20180304_1627",
                                 "/lustre/nyx/cbm/prod/beamtime/2018/02/cosy/"
                                 "r0085_20180304_1721/r0085_20180304_1721",
                                 "/lustre/nyx/cbm/prod/beamtime/2018/02/cosy/"
                                 "r0086_20180304_1729/r0086_20180304_1729",
                                 "/lustre/nyx/cbm/prod/beamtime/2018/02/cosy/"
                                 "r0087_20180304_1753/r0087_20180304_1753",
                                 "/lustre/nyx/cbm/prod/beamtime/2018/02/cosy/"
                                 "r0088_20180304_1807/r0088_20180304_1807",
                                 "/lustre/nyx/cbm/prod/beamtime/2018/02/cosy/"
                                 "r0089_20180304_1826/r0089_20180304_1826"};
  Int_t iTsaFileNb[kiNbRuns]  = {1, 1,  1,  3,  3,  2, 2, 14, 41, 26, 37, 25, 45, 13, 36, 7,  4,  6,  6, 11, 10, 12, 11,
                                8, 11, 26, 1,  8,  9, 8, 3,  3,  22, 17, 6,  4,  5,  4,  1,  5,  5,  5, 9,  13, 27, 45,
                                6, 9,  4,  10, 5,  4, 3, 5,  4,  5,  7,  56, 34, 13, 13, 18, 6,  10, 7, 6,  8,  9,  3,
                                8, 10, 11, 11, 11, 6, 6, 8,  6,  6,  9,  1,  3,  39, 44, 15, 35, 6,  7, 10};

  Double_t dMeanH1[kiNbRuns] = {
    0.835,  0.000,  0.000,  0.566,  0.231,  0.934,  -0.454, -0.495, -0.752, -0.775, 0.000,  -0.799, -0.827,
    0.000,  0.000,  0.000,  0.000,  -1.035, -0.765, -0.414, -0.478, -0.463, -0.481, 0.000,  -0.460, -0.467,
    0.000,  -0.468, -0.444, -0.457, 0.000,  -0.523, -0.460, -0.457, -0.473, -0.487, -0.481, -0.472, -0.477,
    -0.491, -0.454, -0.490, -0.489, -0.469, 0.000,  -0.549, -0.495, -0.475, -0.493, -0.487, -0.474, -0.490,
    -0.484, -0.462, -0.478, -0.478, -0.459, -0.469, 0.000,  -0.498, -0.493, -0.486, -0.497, -0.509, -0.503,
    -0.501, -0.504, -0.492, 0.000,  -0.483, -0.500, 0.000,  -0.492, -0.506, -0.509, -0.520, -0.494, -0.516,
    -0.494, -0.520, -0.502, -0.502, -0.510, -0.498, 0.000,  -0.450, -0.498, -0.514, -0.527};

  Double_t dMeanH2[kiNbRuns] = {
    0.000, 1.816, 0.000, 1.408, 1.270, 1.203, 2.241, 2.363, 2.631, 2.655, 0.000, 2.577, 2.571, 0.000, 0.000,
    0.000, 0.000, 3.163, 3.134, 1.756, 1.847, 1.794, 1.876, 0.000, 1.820, 1.832, 1.793, 1.825, 1.774, 1.771,
    0.000, 2.114, 1.842, 1.867, 1.856, 1.876, 1.877, 1.862, 1.855, 1.864, 1.829, 1.853, 1.877, 1.834, 0.000,
    1.058, 1.108, 1.112, 1.124, 1.114, 1.117, 1.111, 1.109, 1.120, 1.108, 1.117, 1.120, 1.116, 1.126, 1.133,
    1.132, 1.134, 1.131, 1.119, 1.128, 1.129, 1.126, 1.124, 0.000, 1.142, 1.139, 0.000, 1.143, 1.139, 1.143,
    1.147, 1.137, 1.144, 1.128, 1.148, 1.109, 1.133, 1.135, 1.126, 0.000, 1.053, 1.097, 1.097, 1.100};

  Double_t dMeanS1[kiNbRuns] = {
    -54.008, 215.141, 340.020, 10.039,  28.322,  -17.202, 0.000,   0.000,   0.000,   0.000,   0.000,   0.000,   0.000,
    0.000,   0.000,   0.000,   0.000,   0.000,   0.000,   -47.342, -3.632,  -23.295, 0.000,   0.000,   5.389,   -59.796,
    0.000,   596.462, 14.912,  -34.517, 0.000,   2.288,   -3.529,  -53.518, -4.235,  27.358,  31.907,  -48.984, -47.009,
    1.182,   -5.236,  -5.711,  -30.492, -24.589, 0.000,   15.326,  27.849,  33.998,  2.955,   -6.902,  30.775,  -35.417,
    46.218,  21.412,  -47.191, -15.728, 15.353,  -9.277,  21.244,  33.781,  -41.035, -16.084, -16.027, -53.578, -3.440,
    2.616,   -28.384, 27.696,  0.000,   -15.670, -15.665, 0.000,   21.170,  39.868,  -16.071, -3.502,  39.925,  -9.861,
    8.643,   0.000,   0.000,   33.949,  2.567,   -9.634,  0.000,   14.437,  46.383,  14.951,  27.304};

  Double_t dMeanS2[kiNbRuns] = {
    0.000,   0.000,   0.000,   0.000, 0.000,  0.000,   0.000,  0.000,   0.000,   0.000,  0.000,   0.000,  0.000,
    0.000,   0.000,   0.000,   0.000, 0.000,  0.000,   0.000,  0.000,   0.000,   0.000,  0.000,   0.000,  0.000,
    0.000,   0.000,   0.000,   0.000, 0.000,  0.000,   0.000,  0.000,   0.000,   0.000,  0.000,   0.000,  0.000,
    0.000,   0.000,   0.000,   0.000, 0.000,  0.000,   0.000,  0.000,   0.000,   0.000,  0.000,   0.000,  0.000,
    0.000,   0.000,   0.000,   0.000, 0.000,  0.000,   14.290, -16.586, 7.923,   39.063, -23.030, 14.045, 1.833,
    -16.821, -48.145, -23.131, 0.000, 32.746, 32.762,  0.000,  1.743,   -41.798, 32.961, -16.588, 14.649, -3.452,
    -4.112,  -35.387, -41.543, 1.796, 14.283, -29.142, 0.000,  0.000,   -3.814,  8.407,  39.844};

  Double_t dMeanH1H2[kiNbRuns] = {
    0.000,   0.000,   0.000,   32.400,  -14.070, 19.970,  -3.308, 30.151,  30.088,  -33.443, 0.000,  2.029,  30.047,
    0.000,   0.000,   0.000,   0.000,   6.681,   56.669,  29.235, -2.666,  17.737,  -7.889,  0.000,  -7.885, -13.656,
    0.000,   856.227, 23.571,  17.507,  0.000,   -51.621, 1.729,  49.196,  -8.096,  -14.954, 12.193, 1.754,  -33.677,
    -8.351,  -39.139, -52.376, -20.808, 36.537,  0.000,   0.957,  -34.787, 42.017,  -3.981,  5.750,  17.437, -22.329,
    -40.883, 17.267,  10.988,  -47.152, 5.620,   17.345,  0.000,  0.777,   29.297,  17.072,  23.206, 11.057, -10.061,
    29.280,  0.723,   41.668,  0.000,   -28.312, -28.471, 0.000,  29.300,  -41.039, 17.294,  5.512,  11.025, 23.273,
    0.814,   17.169,  -3.873,  -40.982, 35.517,  10.878,  0.000,  5.054,   29.163,  23.090,  -15.980};

  Double_t dMeanS1S2[kiNbRuns] = {
    0.000,  0.000,   0.000,   0.000,  0.000,  0.000,   0.000,   0.000,  0.000,   0.000,   0.000,   0.000,  0.000,
    0.000,  0.000,   0.000,   0.000,  0.000,  0.000,   0.000,   0.000,  0.000,   0.000,   0.000,   0.000,  0.000,
    0.000,  0.000,   0.000,   0.000,  0.000,  0.000,   0.000,   0.000,  0.000,   0.000,   0.000,   0.000,  0.000,
    0.000,  0.000,   0.000,   0.000,  0.000,  0.000,   0.000,   0.000,  0.000,   0.000,   0.000,   0.000,  0.000,
    0.000,  0.000,   0.000,   0.000,  0.000,  0.000,   -29.896, 18.786, 8.916,   7.633,   -22.511, -7.085, 16.583,
    -4.589, -16.567, 15.186,  0.000,  -1.691, -1.697,  0.000,   31.677, -20.676, 6.086,   -35.163, 5.983,  -26.909,
    -7.083, 0.000,   -13.234, -0.974, -3.358, -31.028, 0.000,   7.157,  -29.260, -25.548, 4.872};

  Double_t dMeanH1H2S1[kiNbRuns] = {
    0.000,   0.000,   0.000,   0.000,   11.066,  -3.296, 0.000,  0.000,   11.501, 2.346,   0.000,   -10.852, 0.000,
    0.000,   0.000,   0.000,   0.000,   0.000,   0.000,  7.609,  -38.836, 15.554, -3.253,  0.000,   -3.492,  4.635,
    90.431,  394.434, -9.282,  -3.968,  0.000,   0.000,  19.328, -7.139,  -0.899, -18.120, -35.238, 14.503,  0.576,
    -26.310, 4.192,   18.773,  17.911,  16.287,  0.000,  18.630, 1.889,   -0.210, 8.285,   -28.632, -23.174, 7.925,
    2.238,   -32.095, -10.066, -11.681, -29.604, 19.908, -4.256, -18.185, -9.587, -26.747, 3.419,   -3.379,  -19.343,
    19.127,  -9.213,  -26.207, 0.000,   -0.304,  -0.238, 0.000,  -14.699, 3.319,  -26.856, -8.578,  -20.805, 11.574,
    -11.596, 0.000,   5.890,   -3.442,  -3.115,  0.590,  0.000,  -10.953, -9.355, 2.232,   -29.664};

  Double_t dMeanH1H2S2[kiNbRuns] = {
    0.000,   0.000,   0.000,   0.000,  0.000,  0.000,   0.000,   0.000,  0.000,   0.000,   0.000,   0.000,   0.000,
    0.000,   0.000,   0.000,   0.000,  0.000,  0.000,   0.000,   0.000,  0.000,   0.000,   0.000,   0.000,   0.000,
    0.000,   0.000,   0.000,   0.000,  0.000,  0.000,   0.000,   0.000,  0.000,   0.000,   0.000,   0.000,   0.000,
    0.000,   0.000,   0.000,   0.000,  0.000,  0.000,   0.000,   0.000,  0.000,   0.000,   0.000,   0.000,   0.000,
    0.000,   0.000,   0.000,   0.000,  0.000,  0.000,   -13.789, -0.770, -2.750,  -22.673, -19.766, -13.304, -2.177,
    13.825,  -25.721, -10.631, 0.000,  -2.380, -2.417,  0.000,   16.462, -14.499, -20.486, -45.494, -14.946, -13.723,
    -22.513, -0.670,  2.175,   -5.675, -8.093, -28.340, 0.000,   0.204,  -38.880, -23.861, -26.020};

  Double_t dMeanH1H2S1S2[kiNbRuns] = {
    6.614,   0.000,   0.000,   0.000,  0.000,   0.802,   0.000,  0.000,   0.000,   1.981,   0.000,   -4.622,  0.000,
    0.000,   0.000,   0.000,   0.000,  0.000,   0.000,   -2.868, -16.035, 4.588,   -0.759,  0.000,   0.367,   -2.474,
    46.388,  -80.700, -0.656,  -1.882, 0.000,   8.575,   10.570, 0.577,   1.228,   -5.077,  -13.596, -0.161,  0.524,
    -9.060,  10.302,  20.753,  13.953, 5.686,   0.000,   9.109,  3.685,   -6.563,  -3.057,  -8.883,  -9.890,  2.263,
    1.473,   -12.397, -5.858,  0.781,  -13.137, 9.386,   -7.374, -9.450,  -6.200,  -24.826, -8.134,  -8.285,  -10.283,
    15.890,  -17.143, -17.256, 0.000,  -1.188,  -1.093,  0.000,  -0.120,  -5.703,  -22.827, -25.594, -17.566, -0.920,
    -16.743, -1.366,  -0.989,  -5.011, -5.544,  -13.448, 0.000,  -4.869,  -23.172, -10.533, -26.231};

  Double_t dSigmaH1[kiNbRuns] = {
    10.821, 0.000, 0.000, 9.583, 9.703, 9.581, 9.237, 9.376, 9.564, 9.461, 0.000, 9.573, 9.661, 0.000, 0.000,
    0.000,  0.000, 8.941, 9.140, 6.625, 6.385, 6.470, 6.444, 0.000, 6.417, 6.463, 0.000, 6.413, 6.516, 6.499,
    0.000,  6.430, 6.416, 6.436, 6.369, 6.355, 6.337, 6.381, 6.383, 6.366, 6.431, 6.335, 6.332, 6.413, 0.000,
    5.046,  5.462, 5.576, 5.562, 5.523, 5.539, 5.519, 5.538, 5.593, 5.714, 5.509, 5.560, 5.566, 0.000, 5.597,
    5.611,  5.590, 5.593, 5.598, 5.574, 5.722, 5.611, 5.633, 0.000, 5.681, 5.613, 0.000, 5.604, 5.610, 5.574,
    5.537,  5.634, 5.621, 5.678, 5.553, 5.713, 5.600, 5.637, 5.662, 0.000, 5.953, 5.678, 5.657, 5.689};

  Double_t dSigmaH2[kiNbRuns] = {
    0.000, 11.463, 0.000, 9.045, 9.118, 9.001, 9.088, 9.017, 9.135, 9.049, 0.000, 9.147, 9.205, 0.000, 0.000,
    0.000, 0.000,  9.202, 8.768, 7.164, 7.047, 7.103, 7.081, 0.000, 7.058, 7.096, 7.048, 7.051, 7.134, 7.120,
    0.000, 7.666,  7.066, 7.067, 7.020, 7.027, 7.009, 7.050, 7.039, 7.024, 7.073, 7.024, 7.018, 7.079, 0.000,
    4.868, 5.116,  5.209, 5.208, 5.172, 5.193, 5.174, 5.186, 5.219, 5.310, 5.170, 5.210, 5.204, 5.204, 5.222,
    5.233, 5.215,  5.214, 5.221, 5.200, 5.306, 5.226, 5.251, 0.000, 5.282, 5.238, 0.000, 5.237, 5.238, 5.208,
    5.191, 5.259,  5.243, 5.284, 5.203, 5.319, 5.240, 5.256, 5.271, 0.000, 5.488, 5.288, 5.276, 5.301};

  Double_t dSigmaS1[kiNbRuns] = {
    5.362, 5.164, 11.797, 4.510, 4.657, 6.761, 0.000, 0.000, 0.000, 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
    0.000, 0.000, 0.000,  0.000, 4.094, 4.094, 4.661, 0.000, 0.000, 5.686, 4.307, 0.000, 3.973, 4.035, 3.802,
    0.000, 3.640, 3.923,  3.980, 5.595, 4.561, 6.945, 7.266, 6.496, 6.145, 6.792, 6.765, 7.380, 7.788, 0.000,
    3.636, 3.385, 17.066, 7.545, 7.529, 5.354, 4.163, 4.077, 4.082, 3.946, 3.875, 3.642, 4.293, 3.811, 3.809,
    3.815, 4.350, 4.175,  4.144, 3.625, 3.550, 3.417, 3.293, 0.000, 3.230, 3.225, 0.000, 4.318, 4.250, 4.363,
    4.216, 4.257, 4.425,  3.938, 0.000, 0.000, 4.224, 4.298, 4.753, 0.000, 5.976, 4.601, 4.875, 4.323};

  Double_t dSigmaS2[kiNbRuns] = {
    0.000, 0.000, 0.000, 0.000, 0.000, 0.000, 0.000, 0.000, 0.000, 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
    0.000, 0.000, 0.000, 0.000, 0.000, 0.000, 0.000, 0.000, 0.000, 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
    0.000, 0.000, 0.000, 0.000, 0.000, 0.000, 0.000, 0.000, 0.000, 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,
    0.000, 0.000, 0.000, 0.000, 0.000, 0.000, 0.000, 0.000, 0.000, 0.000, 0.000, 0.000, 0.000, 3.984, 4.005,
    4.064, 4.536, 4.333, 4.622, 4.294, 4.600, 4.543, 4.540, 0.000, 4.504, 4.481, 0.000, 4.081, 3.970, 3.802,
    3.637, 3.584, 3.479, 4.471, 4.469, 4.528, 4.126, 4.404, 4.463, 0.000, 0.000, 3.966, 3.736, 3.415};

  Double_t dSigmaH1H2[kiNbRuns] = {
    0.000, 0.000, 0.000, 8.261, 8.140, 8.106, 6.898, 8.428, 8.894, 8.445, 0.000, 6.806, 9.091, 0.000, 0.000,
    0.000, 0.000, 6.867, 7.064, 8.032, 5.433, 6.483, 6.571, 0.000, 6.489, 7.080, 0.000, 5.496, 6.977, 6.707,
    0.000, 5.954, 4.814, 5.608, 6.362, 6.225, 5.650, 4.762, 6.186, 6.140, 6.560, 5.941, 6.371, 5.858, 0.000,
    3.483, 4.569, 4.922, 4.477, 4.202, 5.373, 5.320, 4.868, 5.497, 5.313, 4.709, 4.264, 5.398, 0.000, 3.958,
    5.300, 5.458, 5.428, 4.963, 5.134, 5.708, 3.962, 5.097, 0.000, 5.830, 5.402, 0.000, 5.247, 5.096, 5.307,
    4.166, 5.134, 5.460, 4.069, 5.328, 4.895, 5.154, 5.095, 5.204, 0.000, 5.024, 5.656, 5.828, 5.970};

  Double_t dSigmaS1S2[kiNbRuns] = {
    0.000, 0.000,  0.000, 0.000,  0.000, 0.000, 0.000, 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,  0.000, 0.000,
    0.000, 0.000,  0.000, 0.000,  0.000, 0.000, 0.000, 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,  0.000, 0.000,
    0.000, 0.000,  0.000, 0.000,  0.000, 0.000, 0.000, 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,  0.000, 0.000,
    0.000, 0.000,  0.000, 0.000,  0.000, 0.000, 0.000, 0.000, 0.000, 0.000, 0.000, 0.000, 0.000,  6.092, 5.103,
    5.252, 10.401, 9.165, 12.259, 7.186, 6.004, 5.673, 5.422, 0.000, 5.226, 5.159, 0.000, 10.490, 6.672, 5.573,
    5.717, 5.407,  5.129, 7.170,  0.000, 3.391, 6.422, 5.339, 7.330, 0.000, 6.131, 6.216, 7.828,  5.320};

  Double_t dSigmaH1H2S1[kiNbRuns] = {
    0.000,  0.000,  0.000,  0.000,  16.608, 6.708,  0.000,  0.000,  5.542,  4.312,  0.000,  2.602,  0.000,
    0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  12.267, 7.721,  10.360, 2.927,  0.000,  6.824,  9.343,
    3.316,  5.175,  9.003,  6.406,  0.000,  0.000,  9.731,  5.824,  6.055,  7.292,  8.431,  8.232,  6.591,
    8.820,  8.798,  6.920,  8.615,  8.692,  0.000,  9.033,  4.638,  29.512, 18.926, 16.044, 11.688, 7.911,
    8.187,  12.874, 11.488, 10.906, 12.482, 12.456, 2.697,  12.616, 11.921, 14.320, 8.107,  11.167, 13.261,
    13.248, 10.492, 14.711, 0.000,  13.034, 11.693, 0.000,  12.583, 11.695, 13.481, 10.835, 13.060, 10.465,
    13.143, 0.000,  3.362,  11.527, 12.638, 11.222, 0.000,  15.346, 12.442, 11.804, 15.400};

  Double_t dSigmaH1H2S2[kiNbRuns] = {
    0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000, 0.000, 0.000,  0.000,  0.000,  0.000,  0.000,
    0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000, 0.000, 0.000,  0.000,  0.000,  0.000,  0.000,
    0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000, 0.000, 0.000,  0.000,  0.000,  0.000,  0.000,
    0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000, 0.000, 0.000,  0.000,  0.000,  0.000,  0.000,
    0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  4.550, 6.328, 10.607, 12.435, 12.198, 12.180, 10.880,
    11.817, 12.238, 12.022, 0.000,  11.882, 10.384, 0.000, 9.884, 10.461, 10.909, 13.252, 13.249, 12.415,
    13.226, 6.806,  10.882, 10.909, 9.372,  12.233, 0.000, 2.322, 12.620, 13.161, 13.296};

  Double_t dSigmaH1H2S1S2[kiNbRuns] = {
    29.006, 0.000,  0.000,  0.000,  0.000,  4.717,  0.000, 0.000,  0.000,  1.982,  0.000,  1.694,  0.000,
    0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  3.803, 4.014,  3.553,  1.498,  0.000,  1.932,  1.620,
    1.791,  1.359,  2.645,  1.936,  0.000,  1.536,  3.400, 5.509,  1.989,  2.469,  4.110,  1.248,  1.308,
    3.932,  4.129,  4.409,  5.830,  2.935,  0.000,  3.161, 4.761,  14.294, 10.565, 3.198,  6.509,  2.766,
    1.963,  4.770,  1.855,  5.201,  3.538,  5.071,  8.342, 11.409, 10.184, 14.581, 6.580,  11.698, 12.281,
    13.504, 12.114, 14.824, 0.000,  11.909, 10.131, 0.000, 9.712,  11.553, 13.653, 14.404, 12.895, 6.267,
    13.471, 2.402,  3.238,  10.383, 10.214, 11.104, 0.000, 3.988,  14.040, 11.541, 15.946};

  Double_t dWidthH1[kiNbRuns] = {
    43.286, 0.000,  0.000,  38.332, 38.812, 38.325, 36.949, 37.506, 38.258, 37.845, 0.000,  38.292, 38.646,
    0.000,  0.000,  0.000,  0.000,  35.765, 36.561, 26.498, 25.539, 25.881, 25.774, 0.000,  25.667, 25.853,
    0.000,  25.650, 26.065, 25.997, 0.000,  25.718, 25.662, 25.742, 25.477, 25.421, 25.349, 25.523, 25.532,
    25.464, 25.726, 25.339, 25.326, 25.652, 0.000,  20.186, 21.847, 22.303, 22.247, 22.091, 22.157, 22.075,
    22.153, 22.370, 22.854, 22.038, 22.240, 22.264, 0.000,  22.390, 22.444, 22.362, 22.373, 22.393, 22.297,
    22.888, 22.443, 22.533, 0.000,  22.726, 22.451, 0.000,  22.417, 22.441, 22.295, 22.150, 22.534, 22.483,
    22.713, 22.211, 22.852, 22.401, 22.548, 22.647, 0.000,  23.810, 22.714, 22.629, 22.758};

  Double_t dWidthH2[kiNbRuns] = {
    0.000,  45.853, 0.000,  36.179, 36.472, 36.003, 36.352, 36.069, 36.541, 36.195, 0.000,  36.589, 36.821,
    0.000,  0.000,  0.000,  0.000,  36.808, 35.073, 28.654, 28.189, 28.411, 28.324, 0.000,  28.232, 28.384,
    28.192, 28.205, 28.537, 28.479, 0.000,  30.665, 28.264, 28.269, 28.079, 28.110, 28.034, 28.199, 28.157,
    28.098, 28.291, 28.095, 28.073, 28.317, 0.000,  19.471, 20.464, 20.835, 20.832, 20.686, 20.771, 20.698,
    20.743, 20.878, 21.238, 20.682, 20.840, 20.817, 20.816, 20.886, 20.933, 20.860, 20.854, 20.883, 20.798,
    21.224, 20.903, 21.005, 0.000,  21.127, 20.951, 0.000,  20.949, 20.950, 20.833, 20.765, 21.038, 20.970,
    21.138, 20.812, 21.276, 20.959, 21.023, 21.084, 0.000,  21.951, 21.151, 21.104, 21.204};

  Double_t dWidthS1[kiNbRuns] = {
    42.896, 41.314, 94.375, 36.083, 37.259, 54.089, 0.000,  0.000,  0.000,   0.000,  0.000,  0.000,  0.000,
    0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  32.752, 32.750, 37.288,  0.000,  0.000,  45.486, 34.457,
    0.000,  31.783, 32.277, 30.415, 0.000,  29.118, 31.383, 31.838, 44.761,  36.489, 55.558, 58.131, 51.965,
    49.159, 54.339, 54.117, 59.036, 62.302, 0.000,  29.090, 27.079, 136.529, 60.362, 60.234, 42.829, 33.307,
    32.616, 32.654, 31.570, 30.996, 29.133, 34.341, 30.487, 30.469, 30.522,  34.801, 33.401, 33.152, 29.001,
    28.400, 27.337, 26.342, 0.000,  25.843, 25.798, 0.000,  34.543, 33.999,  34.904, 33.729, 34.059, 35.403,
    31.501, 0.000,  0.000,  33.792, 34.382, 38.027, 0.000,  47.809, 36.809,  38.997, 34.581};

  Double_t dWidthS2[kiNbRuns] = {
    0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000,
    0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000,
    0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000,
    0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000,
    0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  31.869, 32.043, 32.513, 36.284, 34.667, 36.974, 34.355,
    36.797, 36.345, 36.322, 0.000,  36.035, 35.846, 0.000,  32.650, 31.759, 30.416, 29.095, 28.671, 27.835,
    35.765, 35.751, 36.223, 33.012, 35.229, 35.701, 0.000,  0.000,  31.731, 29.884, 27.318};

  Double_t dWidthH1H2[kiNbRuns] = {
    0.000,  0.000,  0.000,  33.046, 32.562, 32.424, 27.592, 33.714, 35.576, 33.781, 0.000,  27.223, 36.366,
    0.000,  0.000,  0.000,  0.000,  27.468, 28.254, 32.127, 21.730, 25.934, 26.284, 0.000,  25.956, 28.320,
    0.000,  21.983, 27.907, 26.827, 0.000,  23.816, 19.255, 22.434, 25.450, 24.900, 22.599, 19.050, 24.744,
    24.560, 26.240, 23.765, 25.485, 23.431, 0.000,  13.932, 18.276, 19.687, 17.910, 16.809, 21.492, 21.281,
    19.474, 21.990, 21.252, 18.838, 17.055, 21.594, 0.000,  15.832, 21.202, 21.831, 21.713, 19.852, 20.535,
    22.830, 15.849, 20.387, 0.000,  23.318, 21.606, 0.000,  20.987, 20.385, 21.227, 16.665, 20.537, 21.839,
    16.276, 21.311, 19.580, 20.617, 20.380, 20.814, 0.000,  20.096, 22.624, 23.311, 23.880};

  Double_t dWidthS1S2[kiNbRuns] = {
    0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000,
    0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000,
    0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000,
    0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000,
    0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  24.368, 20.413, 21.007, 41.603, 36.659, 49.035, 28.744,
    24.016, 22.692, 21.687, 0.000,  20.904, 20.635, 0.000,  41.959, 26.689, 22.291, 22.868, 21.627, 20.515,
    28.679, 0.000,  13.564, 25.689, 21.357, 29.320, 0.000,  24.524, 24.863, 31.313, 21.280};

  Double_t dWidthH1H2S1[kiNbRuns] = {
    0.000,  0.000,  0.000,  0.000,  66.431, 26.830, 0.000,  0.000,  22.167,  17.248, 0.000,  10.410, 0.000,
    0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  49.069, 30.885, 41.440,  11.709, 0.000,  27.296, 37.373,
    13.263, 20.699, 36.010, 25.623, 0.000,  0.000,  38.925, 23.295, 24.222,  29.167, 33.724, 32.927, 26.363,
    35.279, 35.193, 27.681, 34.459, 34.769, 0.000,  36.133, 18.552, 118.047, 75.705, 64.174, 46.754, 31.645,
    32.747, 51.494, 45.950, 43.626, 49.930, 49.825, 10.788, 50.463, 47.685,  57.280, 32.427, 44.669, 53.044,
    52.992, 41.967, 58.846, 0.000,  52.135, 46.772, 0.000,  50.331, 46.781,  53.923, 43.340, 52.238, 41.858,
    52.574, 0.000,  13.449, 46.106, 50.550, 44.890, 0.000,  61.383, 49.770,  47.217, 61.602};

  Double_t dWidthH1H2S2[kiNbRuns] = {
    0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000,
    0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000,
    0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000,
    0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000,
    0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  18.201, 25.312, 42.429, 49.741, 48.792, 48.719, 43.520,
    47.269, 48.954, 48.087, 0.000,  47.528, 41.537, 0.000,  39.535, 41.845, 43.637, 53.007, 52.995, 49.661,
    52.903, 27.224, 43.526, 43.634, 37.486, 48.932, 0.000,  9.288,  50.482, 52.644, 53.185};

  Double_t dWidthH1H2S1S2[kiNbRuns] = {
    116.022, 0.000,  0.000,  0.000,  0.000,  18.867, 0.000,  0.000,  0.000,  7.927,  0.000,  6.774,  0.000,
    0.000,   0.000,  0.000,  0.000,  0.000,  0.000,  15.212, 16.056, 14.214, 5.991,  0.000,  7.726,  6.481,
    7.165,   5.437,  10.580, 7.742,  0.000,  6.146,  13.599, 22.035, 7.957,  9.875,  16.439, 4.993,  5.233,
    15.727,  16.517, 17.638, 23.321, 11.740, 0.000,  12.644, 19.045, 57.176, 42.258, 12.791, 26.037, 11.063,
    7.852,   19.081, 7.420,  20.803, 14.151, 20.283, 33.369, 45.636, 40.737, 58.323, 26.320, 46.791, 49.126,
    54.016,  48.456, 59.295, 0.000,  47.635, 40.523, 0.000,  38.847, 46.211, 54.613, 57.615, 51.582, 25.067,
    53.882,  9.606,  12.953, 41.532, 40.857, 44.418, 0.000,  15.953, 56.158, 46.164, 63.783};

  Double_t dMeanX_S1[kiNbRuns] = {
    0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000,
    0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  -2.598, -2.601, -2.589, 0.000,  0.000,  -2.584, -2.600,
    0.000,  -2.596, -2.596, -2.596, 0.000,  0.000,  -2.552, -2.552, -2.458, -2.501, -2.466, -2.475, -2.564,
    -2.563, -2.644, -2.751, -2.524, -2.585, 0.000,  -2.479, -2.487, 21.289, -3.134, -2.505, -2.474, -2.476,
    -2.476, -2.477, -2.479, -2.480, -2.479, -2.464, 0.000,  -1.937, -1.893, -0.445, -1.437, -1.738, -1.715,
    -1.048, -1.828, -1.718, 0.000,  -1.652, -1.585, 0.000,  -1.645, -1.676, -1.832, -1.777, -1.821, -0.963,
    -1.034, 0.000,  0.000,  -1.191, -1.156, -0.893, 0.000,  -1.613, -1.006, -1.070, -1.260};

  Double_t dMeanY_S1[kiNbRuns] = {
    0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.000, 0.000,  0.000,    0.000,  0.000,  0.000,  0.000,
    0.000,  0.000,  0.000,  0.000,  0.000,  0.000,  0.118, 0.118,  0.128,    0.000,  0.000,  0.146,  0.139,
    0.000,  0.158,  0.152,  0.153,  0.000,  0.000,  0.253, 0.252,  0.379,    0.334,  0.370,  0.398,  0.316,
    0.331,  0.326,  0.316,  0.406,  0.403,  0.000,  0.371, 0.368,  -207.965, -1.848, 0.444,  0.430,  0.419,
    0.419,  0.417,  0.417,  0.417,  0.417,  0.431,  0.000, -1.502, -0.723,   -0.284, -1.169, -1.334, -1.404,
    -0.782, -1.477, -1.479, 0.000,  -1.485, -1.399, 0.000, -0.801, -1.367,   -1.521, -1.443, -1.489, -0.725,
    -0.779, 0.000,  0.000,  -0.415, -0.733, -0.269, 0.000, -1.296, -0.632,   -0.675, -0.809};

  Double_t dMeanX_S2[kiNbRuns] = {
    0.000,  0.000,  0.000,  0.000, 0.000,  0.000,  0.000, 0.000,  0.000,  0.000,  0.000,  0.000,  0.000,
    0.000,  0.000,  0.000,  0.000, 0.000,  0.000,  0.000, 0.000,  0.000,  0.000,  0.000,  0.000,  0.000,
    0.000,  0.000,  0.000,  0.000, 0.000,  0.000,  0.000, 0.000,  0.000,  0.000,  0.000,  0.000,  0.000,
    0.000,  0.000,  0.000,  0.000, 0.000,  0.000,  0.000, 0.000,  0.000,  0.000,  0.000,  0.000,  0.000,
    0.000,  0.000,  0.000,  0.000, 0.000,  0.000,  0.000, -0.485, -0.487, -0.488, -0.490, -0.491, -0.492,
    -0.492, -0.493, -0.494, 0.000, -0.495, -0.495, 0.000, -0.496, -0.498, -0.498, -0.498, -0.499, -0.499,
    -0.500, 0.015,  0.017,  0.017, 0.015,  0.014,  0.000, 0.000,  0.012,  0.012,  0.014};

  Double_t dMeanY_S2[kiNbRuns] = {
    0.000,  0.000,  0.000,  0.000, 0.000,  0.000,  0.000, 0.000,  0.000,  0.000,  0.000,  0.000,  0.000,
    0.000,  0.000,  0.000,  0.000, 0.000,  0.000,  0.000, 0.000,  0.000,  0.000,  0.000,  0.000,  0.000,
    0.000,  0.000,  0.000,  0.000, 0.000,  0.000,  0.000, 0.000,  0.000,  0.000,  0.000,  0.000,  0.000,
    0.000,  0.000,  0.000,  0.000, 0.000,  0.000,  0.000, 0.000,  0.000,  0.000,  0.000,  0.000,  0.000,
    0.000,  0.000,  0.000,  0.000, 0.000,  0.000,  0.000, 12.151, 12.151, 12.153, 12.155, 12.157, 12.157,
    12.156, 12.158, 12.157, 0.000, 12.182, 12.183, 0.000, 12.182, 12.183, 12.182, 12.182, 12.182, 12.180,
    12.184, 0.050,  0.050,  0.048, 0.050,  0.054,  0.000, 0.000,  0.049,  0.045,  0.040};

  if (iRunIdx <= 0 || kiNbRuns < iRunIdx) return;

  TString inFile  = sFilePath[iRunIdx - 1];
  Int_t iStopFile = iTsaFileNb[iRunIdx - 1];

  // --- Specify number of events to be produced.
  // --- -1 means run until the end of the input file.
  Int_t nEvents = -1;

  // --- Specify output file name (this is just an example)
  TString outFile = Form("data/eff_dryrun_out_r%04u.root", iRunIdx);
  TString parFile = Form("data/eff_dryrun_param_r%04u.root", iRunIdx);

  // --- Set log output levels
  FairLogger::GetLogger();
  gLogger->SetLogScreenLevel("INFO");
  //  gLogger->SetLogScreenLevel("DEBUG");
  gLogger->SetLogVerbosityLevel("LOW");

  // --- Define parameter files
  TList* parFileList = new TList();
  TString paramDir   = "./";

  TString paramFileHodo          = paramDir + "HodoUnpackPar.par";
  TObjString* tutDetDigiFileHodo = new TObjString(paramFileHodo);
  parFileList->Add(tutDetDigiFileHodo);

  TString paramFileSts          = paramDir + "StsUnpackPar.par";
  TObjString* tutDetDigiFileSts = new TObjString(paramFileSts);
  parFileList->Add(tutDetDigiFileSts);

  // --- Set debug level
  gDebug = 0;

  std::cout << std::endl;
  std::cout << ">>> Cern2017Monitor: output file is " << outFile << std::endl;

  // ========================================================================
  // ========================================================================

  std::cout << std::endl;
  std::cout << ">>> Cern2017Monitor: Initialising..." << std::endl;

  // Hodoscopes Monitor
  CbmCosy2018MonitorEfficiency* monitorEff = new CbmCosy2018MonitorEfficiency();
  monitorEff->SetHistoFileName(Form("data/EfficiencyHistos_dryrun_r%04u.root", iRunIdx));
  //  monitorEff->SetPrintMessage();
  monitorEff->SetMsOverlap();
  monitorEff->EnableDualStsMode(kTRUE);
  //  monitorEff->SetLongDurationLimits( 3600, 10 );
  monitorEff->SetLongDurationLimits(7200, 60);
  /*
  monitorEff->SetCoincidenceBorderHodo(   0.0,  50 );
  monitorEff->SetCoincidenceBorderSts1(   0.0,  75 );
  monitorEff->SetCoincidenceBorderSts2(   0.0,  75 );
*/
  monitorEff->SetCoincidenceBorderHodo(0.0, 100.0);
  monitorEff->SetCoincidenceBorderHodoBoth(0.0, 100.0);
  monitorEff->SetCoincidenceBorderSts1(0.0, 150.0);
  monitorEff->SetCoincidenceBorderSts2(0.0, 150.0);
  monitorEff->SetCoincidenceBorderHodoSts1(0.0, 200.0);
  monitorEff->SetCoincidenceBorderHodoSts2(0.0, 200.0);
  monitorEff->SetCoincidenceBorder(0.0, 200.0);
  monitorEff->SetStripsOffset1(-64, -61);
  monitorEff->SetStripsOffset2(-64, -61);
  monitorEff->SetPositionsMmZ(1230.0, 1620.0, 1350.0, 1470.0);
  monitorEff->SetPositionOffsetSts1(0.0, 0.0);
  monitorEff->SetPositionOffsetSts2(0.0, 0.0);
  monitorEff->EnableDeadCorr(kFALSE);

  // --- Source task
  CbmTofStar2018Source* source = new CbmTofStar2018Source();
  if ("" != inFile) {
    for (Int_t iFileIdx = 0; iFileIdx < iStopFile; ++iFileIdx) {
      TString sFilePath = Form("%s_%04u.tsa", inFile.Data(), iFileIdx);
      source->AddFile(sFilePath);
      std::cout << "Added " << sFilePath << std::endl;
    }  // for( Int_t iFileIdx = iStartFile; iFileIdx < iStopFile; ++iFileIdx )
  }    // if( "" != inFile )
  else {
    source->SetHostName("cbmin002");
    source->SetPortNumber(5556);
  }

  source->AddUnpacker(monitorEff, 0x10, 6);  // stsXyter DPBs

  // --- Event header
  FairEventHeader* event = new CbmTbEvent();
  event->SetRunId(1);

  // --- Run
  run = new FairRunOnline(source);
  run->SetOutputFile(outFile);
  run->SetEventHeader(event);
  //  run->ActivateHttpServer( iServerRefreshRate, iServerHttpPort ); // refresh each 100 events
  run->SetAutoFinish(kFALSE);

  // -----   Runtime database   ---------------------------------------------
  FairRuntimeDb* rtdb       = run->GetRuntimeDb();
  Bool_t kParameterMerged   = kTRUE;
  FairParRootFileIo* parOut = new FairParRootFileIo(kParameterMerged);
  FairParAsciiFileIo* parIn = new FairParAsciiFileIo();
  parOut->open(parFile.Data());
  parIn->open(parFileList, "in");
  rtdb->setFirstInput(parIn);
  rtdb->setOutput(parOut);

  run->Init();

  // --- Start run
  TStopwatch timer;
  timer.Start();
  std::cout << ">>> Cern2017Monitor: Starting run..." << std::endl;
  run->Run(nEvents, 0);  // run until end of input file
  timer.Stop();

  std::cout << "Processed " << std::dec << source->GetTsCount() << " timeslices" << std::endl;

  run->Finish();

  // --- End-of-run info
  Double_t rtime = timer.RealTime();
  Double_t ctime = timer.CpuTime();
  std::cout << std::endl << std::endl;
  std::cout << ">>> Cern2017Monitor: Macro finished successfully." << std::endl;
  std::cout << ">>> Cern2017Monitor: Output file is " << outFile << std::endl;
  std::cout << ">>> Cern2017Monitor: Real time " << rtime << " s, CPU time " << ctime << " s" << std::endl;
  std::cout << std::endl;

  /// --- Screen output for automatic tests
  std::cout << " Test passed" << std::endl;
  std::cout << " All ok " << std::endl;
}
